@@ -28,7 +28,7 @@ Device <- R7Class(
         s <- paste0(s, ", index=", self$index, ")")
       }
 
-      cat(s)
+      cat(s, "\n")
     }
   ),
   active = list(
@@ -50,6 +50,15 @@ Device <- R7Class(
   )
 )
 
+#' @export
+as.character.torch_device <- function(x, ...) {
+  chr <- x$type
+  if (!is.null(x$index)) {
+    chr <- paste0(chr, ":", x$index)
+  }
+  chr
+}
+
 #' Create a Device object
 #'
 #' A `torch_device`  is an object representing the device on which a `torch_tensor`
@@ -125,7 +134,7 @@ is_meta_device <- function(x) {
 local_device <- function(device, ..., .env = parent.frame()) {
   current_device <- cpp_get_current_default_device()
   cpp_set_default_device(device)
-  
+
   withr::defer({
     cpp_set_default_device(current_device)
   }, envir = .env)
