@@ -180,11 +180,6 @@ dim.torch_tensor <- function(x) {
   cpp_tensor_dim(x$ptr)
 }
 
-#' @export
-length.torch_tensor <- function(x) {
-  prod(dim(x))
-}
-
 #' @export
 as.numeric.torch_tensor <- function(x, ...) {
   as.numeric(as_array(x))
