@@ -26,6 +26,10 @@ void delete_vector_string(void* x);
 void delete_string(void* x);
 void delete_stack(void* x);
 void delete_ivalue(void* x);
+void delete_function_schema(void* x);
+void delete_function_schema_list(void* x);
+void delete_function_schema_argument(void* x);
+void delete_function_schema_argument_list(void* x);
 void delete_tuple(void* x);
 void delete_vector_bool(void* x);
 void delete_vector_scalar(void* x);
